searchData={"items":[{"type":"module","title":"Condition","doc":"","ref":"Condition.html"},{"type":"module","title":"ConditionSet","doc":"","ref":"ConditionSet.html"},{"type":"module","title":"Rule","doc":"","ref":"Rule.html"},{"type":"function","title":"Rule.new/1","doc":"","ref":"Rule.html#new/1"},{"type":"module","title":"Rulex","doc":"Documentation for `Rulex`.","ref":"Rulex.html"},{"type":"function","title":"Rulex.evaluate/1","doc":"evaluate/1 function consumes a map with schema:\n```json\n  {\n    \"data\": {},\n    \"rules\": []\n  }\n```","ref":"Rulex.html#evaluate/1"},{"type":"function","title":"Rulex.evaluate/2","doc":"evaluate/2  consumes a json data payload + an array of rules","ref":"Rulex.html#evaluate/2"},{"type":"function","title":"Rulex.stream_rules/2","doc":"","ref":"Rulex.html#stream_rules/2"},{"type":"extras","title":"Rulex","doc":"# Rulex\nAn Elixir rule engine where rules are json objects.","ref":"readme.html"},{"type":"extras","title":"What and why are rule engines useful? - Rulex","doc":"A rules engine is a flexible piece of software that manages business rules.\n\nRule = Condition + Action\n\nThink of business rules as dynamically loaded “if-then” statements. A rules engine fits really well with event hooks, event handling, and ETL flows. Generic webhook endpoint → rule match some condition → route / handle payload.\n\nA general design pattern:\n- collect / observe incoming data\n- evaluate against rules for any matches\n- trigger reactionary actions\n\n**The main benefits:**\n\nIf you store rules in a DB, you can change your business logic on the fly during runtime. Essentially hot swap your business logic without reloading an app or changing code. It also makes changes much more maintainable with faster turn around time.\n\nAnother understated benefit; you can isolate team and company concerns. One team / group can manage business rule CRUD lifecycle while another specialized dev team manages execution logic (action flows and handlers etc). None-coders can create new rules through a form gui and quickly change automations.","ref":"readme.html#what-and-why-are-rule-engines-useful"},{"type":"extras","title":"Installation - Rulex","doc":"[Available in Hex](https://hex.pm/packages/rulex), the package can be installed\nby adding `rulex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:roullexe, \">= 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start Example - Rulex","doc":"[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fraw.githubusercontent.com%2Fgondwe%2Frulex%2Fmain%2Fnotebook%2Fquick_start.livemd)\n\n\n```elixir\niex> payload = to_string('\n{\n    \"data\": {\n        \"person\": {\n            \"name\": \"Lionel\",\n            \"last_name\": \"Messi\",\n            \"interests\": [\n                \"soccer\",\n                \"hot dogs\",\n                \"sports\"\n            ]\n        }\n    },\n    \"rules\": [\n        {\n            \"id\": \"123456\",\n            \"conditions\": {\n                \"all\": [\n                    {\n                        \"path\": \"/person/name\",\n                        \"operator\": \"equals\",\n                        \"value\": \"Lionel\"\n                    },\n                    {\n                        \"path\": \"/person/last_name\",\n                        \"operator\": \"like\",\n                        \"value\": \"mess\"\n                    },\n                    {\n                        \"path\": \"/person/interests\",\n                        \"operator\": \"contains\",\n                        \"value\": \"soccer\"\n                    }\n                ]\n            },\n            \"action\": \"collect_signature.exs\"\n        }\n    ]\n}')\niex>\niex> results = Rulex.evaluate(payload)\niex>\niex> [\n  %{\n    \"action\" => \"collect_signature.exs\",\n    \"conditions\" => %{\n      \"all\" => [\n        %{\"operator\" => \"equals\", \"path\" => \"/person/name\", \"value\" => \"Lionel\"},\n        %{\n          \"operator\" => \"like\",\n          \"path\" => \"/person/last_name\",\n          \"value\" => \"mess\"\n        },\n        %{\n          \"operator\" => \"contains\",\n          \"path\" => \"/person/interests\",\n          \"value\" => \"soccer\"\n        }\n      ]\n    },\n    \"id\" => \"123456\"\n  }\n]\n```\nNotes:\n- Returns a list of matched rules with the complete rule json\n- Rulex.evaluate/1 and Rulex.evaluate/2 take elixir native data format or json binary strings\n- charlist is used for readability here and converted to binary string via to_string()","ref":"readme.html#quick-start-example"},{"type":"extras","title":"Documention and Usage - Rulex","doc":"Docs can be found at  \n\nRulex is storage and action agnostic. Ideally you can store/load rules from a DB and evaluate on incoming json payloads. Handler code can then evaluate rules and action however you like.","ref":"readme.html#documention-and-usage"},{"type":"extras","title":"Rule Schema - Rulex","doc":"The rule schema is very flexible and the only strict requirements are on the conditions. You can add your own rule id and action schema. If a rule is matched, the entire rule object is appended to the result list.\n\n\nExample:\n```json\n{\n    \"id\": \"\",\n    \"conditions\": {},\n    \"action\": {}\n}\n```\nExample:\n```json\n{\n    \"id\": \"d6f05047-807d-4970-b411-5575fb739dda\",\n    \"conditions\": {},\n    \"action\": \"on_match_script.exs\",\n    \"meta_data\": {}\n}\n```","ref":"readme.html#rule-schema"},{"type":"extras","title":"Conditions - Rulex","doc":"Conditions consist of a path, operator, and value.\nExample:\n```json\n{\n    \"path\": \"/person/name\",\n    \"operator\": \"contains\",\n    \"value\": \"ABC\"\n}\n```","ref":"readme.html#conditions"},{"type":"extras","title":"Path - Rulex","doc":"A [rfc json-pointer](https://www.rfc-editor.org/rfc/rfc6901) for the supplied json payload.","ref":"readme.html#path"},{"type":"extras","title":"Operators - Rulex","doc":"- equals\n- not_equal\n- greater_than\n- less_than\n- contains\n- not_contains\n- like\n- regex_match","ref":"readme.html#operators"},{"type":"extras","title":"Value - Rulex","doc":"The value to match or operate on","ref":"readme.html#value"},{"type":"extras","title":"Condition sets - Rulex","doc":"Features:\n- all (and) condition sets\n- any (or) condition sets\n- sets can be nested\n\nExample:\n```json\n{\n    \"any\": [\n        {\n            \"path\": \"/source\",\n            \"operator\": \"contains\",\n            \"value\": \"X\"\n        },\n        {\n            \"path\": \"/product\",\n            \"operator\": \"contains\",\n            \"value\": \"Y\"\n        }\n    ]\n}\n```\nExample:\n```json\n{\n    \"all\": [\n        {\n            \"path\": \"/name\",\n            \"operator\": \"equals\",\n            \"value\": \"A\"\n        },\n        {\n            \"any\": [\n                {\n                    \"path\": \"/source\",\n                    \"operator\": \"contains\",\n                    \"value\": \"X\"\n                },\n                {\n                    \"path\": \"/product\",\n                    \"operator\": \"contains\",\n                    \"value\": \"Y\"\n                }\n            ]\n        }\n    ]\n}\n```","ref":"readme.html#condition-sets"},{"type":"extras","title":"Credits - Rulex","doc":"This project is inspired by:\n- https://github.com/santalvarez/python-rule-engine\n- https://github.com/CacheControl/json-rules-engine","ref":"readme.html#credits"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}